#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <getopt.h>

// lunghezza massima per una singola lettura di linea con fgets
#define DEFAULT_MAX_LINE 2048

// questa variabile e' definita nella libc ed è settata dalla 
// funzione getopt 
extern int optind; // NOTA: non e' necessario ridichiarare la variabile extern qui ... 

// ritorna il numero di parole contenute nella stringa buffer
int wcount(const char buf[]) {
    char *p = (char*)&buf[0];
    int count = 0, flag=0;
    // togliamo tutti gli spazi ed i caratteri di controllo
    // iniziali in modo da iniziare con un carattere
    while(*p != '\0' && (iscntrl(*p) || isspace(*p))) ++p;

    while(*p != '\0') { 
	if (isspace(*p)) {  // se ho uno spazio e se il carattere precedente 
	                    // non e' ne uno spazio ne un carattere di controllo....
	    if ( !isspace(*(p-1)) && !iscntrl(*(p-1)) ) { 
		++count;
		flag = 0;
	    }
	} else flag=1;  // ho iniziato una nuova parola
	++p;
    }
    return (flag) ? count+1:count;
}

int main(int argc, char *argv[]) {
    if (argc == 1) {
	fprintf(stderr, "usa: %s [-l -w -m <num> ] <nome-file> [<nome-file> ...]\n", argv[0]);
	fprintf(stderr, " -l conta il numero di linee\n");
	fprintf(stderr, " -w conta il numero di parole\n");
	fprintf(stderr, " -m <num> setta la lunghezza massima di una linea a <num>\n");
	exit(EXIT_FAILURE);
    }
    int countline = 0, countword = 0;
    char opt;
    long max_line = DEFAULT_MAX_LINE;
    // parsing della command line
    while((opt = getopt(argc, argv, "lwm:")) != -1) {
	switch(opt) {
	case 'l': {
	    countline = 1;
	} break;
	case 'w': {
	    countword = 1;
	} break;
	case 'm': {
	    max_line = strtol(optarg, NULL, 10);
	} break;
	default: ;
	}
    }

    // se non ci sono opzioni considero sia -l che -w come abilitate
    if (countword + countline == 0) { countline=1; countword=1; }

    char *line = malloc(max_line*sizeof(char)); // buffer
    if (!line) {
	perror("malloc");
	return EXIT_FAILURE;
    }
    // per ogni file 
    while(argv[optind] != NULL) {
	FILE *fp;             // file pointer
	long numline,numword; // contatori
	
	char *filename = argv[optind];
	if ((fp = fopen(filename,"r")) == NULL) {
	    perror("fopen");
	    return EXIT_FAILURE;
	}

	numline=numword=0;
	while(fgets(line, max_line, fp) != NULL) { 
	    if (line[strlen(line)-1] == '\n') {
		++numline;
	    }
	    if (countword) numword += wcount(line);
	}
	fclose(fp);
	if (countline && !countword) printf("  %ld %s\n", numline, argv[optind]);
	if (!countline && countword) printf("  %ld %s\n", numword, argv[optind]);
	if (countline && countword)  printf("  %ld  %ld %s\n", numline, numword, argv[optind]);
	++optind;
    }
    if (line) free(line);

    return EXIT_SUCCESS; // va bene anche return 0;
}
